with
  text_io,
  ada.integer_text_io,
  ada.long_float_text_io,
  ada.numerics,
  ada.numerics.long_elementary_functions;

procedure polygon is

  type vec3 is array (1 .. 3) of long_float;

  type fmat1 is array (integer range <>) of long_float;

  type fmat1_ptr is access fmat1;

  ns,ni,nn              : integer;
  radius,area,thickness : long_float;
  dr1,dz1,dr2,dz2       : long_float;
  cost,sint             : fmat1_ptr;

  procedure scan (io : in text_io.file_type) is

    ascii : character;

  begin
    loop
      text_io.get(io,ascii);
      text_io.skip_line(io);
      exit when ascii = ':';
    end loop;
  end scan;

  function node (i,j : in integer) return integer is

  begin
    if i = 0 and j = 0 then
      return 1;
    else
      return i * (i - 1) / 2 * ns + j mod (i * ns) + 2;
    end if;
  end node;

  procedure vertex (i,j : in integer;
                    x   : out vec3) is

    k       : integer;
    alpha,r : long_float;

  begin
    if i > 0 then
      k := j mod (ns * i) / i;
      alpha := long_float(j mod (ns * i) - k * i) / long_float(i);
      r := long_float(i) / long_float(ni) * radius;
      x(1) := (1.0 - alpha) * r * cost(k) + alpha * r * cost((k + 1) mod ns);
      x(2) := (1.0 - alpha) * r * sint(k) + alpha * r * sint((k + 1) mod ns);
      x(3) := 0.0;
    else
      x(1) := 0.0;
      x(2) := 0.0;
      x(3) := 0.0;
    end if;
  end vertex;

  procedure initia is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io,
      ada.numerics,
      ada.numerics.long_elementary_functions;

    angle : long_float;
    io    : file_type;

  begin
    -- A regular polygon with ns sides is inscribed in a circumference
    -- The radius is divided in ni equal parts
    -- The edges have line elements
    -- Each corner is displaced alternately by (dr1, dz1) or (dr2, dz2)

    open(io, in_file, "polygon.txt");

    -- ns, ni, radius, area, thickness
    scan(io);
    get(io,ns);
    get(io,ni);
    get(io,radius);
    get(io,area);
    get(io,thickness);
    skip_line(io);

    -- dr1, dz1, dr2, dz2
    scan(io);
    get(io,dr1);
    get(io,dz1);
    get(io,dr2);
    get(io,dz2);
    skip_line(io);

    close(io);

    -- nn
    nn := (ni + 1) * ni / 2 * ns + 1;

    -- cost, sint
    cost := new fmat1 (0 .. ns - 1);
    sint := new fmat1 (0 .. ns - 1);
    angle := 2.0 * pi / long_float(ns);
    for k in 0 .. ns - 1 loop
      cost(k) := cos(angle * long_float(k));
      sint(k) := sin(angle * long_float(k));
    end loop;
  end initia;

  procedure input is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io;

    maxls,maxminor,maxlist,maxmajor : integer;
    lstol,maxnlag,maxncon,mu        : long_float;
    e,glob,k                        : integer;
    x                               : vec3;
    io                              : file_type;

  begin
    -- solution parameters
    lstol := 1.0E-01;
    maxls := 20;
    maxminor := 30 * nn;
    maxlist := 3;
    maxnlag := 1.0E-04;
    mu := 0.0;
    maxncon := 0.0;
    maxmajor := 0;

    create(io, out_file, "input.txt");

    put(io,"        lstol     maxls  maxminor   maxlist                mu           maxnlag           maxncon  maxmajor");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,lstol,2,6,3);
    put(io,"     ");
    put(io,maxls,5);
    put(io,"     ");
    put(io,maxminor,5);
    put(io,"     ");
    put(io,maxlist,5);
    put(io,"     ");
    put(io,mu,2,6,3);
    put(io,"     ");
    put(io,maxnlag,2,6,3);
    put(io,"     ");
    put(io,maxncon,2,6,3);
    put(io,"     ");
    put(io,maxmajor,5);
    new_line(io);

    -- node coordinates
    new_line(io);
    new_line(io);
    put(io," node           coord x           coord y           coord z");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,nn,5);
    new_line(io);

    new_line(io);
    vertex(0,0,x);
    put(io,node(0,0),5);
    put(io,"     ");
    put(io,x(1),2,6,3);
    put(io,"     ");
    put(io,x(2),2,6,3);
    put(io,"     ");
    put(io,x(3),2,6,3);
    new_line(io);

    for i in 1 .. ni loop
      for j in 0 .. i * ns - 1 loop
        vertex(i,j,x);
        put(io,node(i,j),5);
        put(io,"     ");
        put(io,x(1),2,6,3);
        put(io,"     ");
        put(io,x(2),2,6,3);
        put(io,"     ");
        put(io,x(3),2,6,3);
        new_line(io);
      end loop;
    end loop;

    -- element 11
    new_line(io);
    new_line(io);
    put(io,"elem1      node      node              area");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,ni * ns,5);
    new_line(io);

    new_line(io);
    e := 0;

    for j in 0 .. ni * ns - 1 loop
      e := e + 1;

      put(io,e,5);
      put(io,"     ");
      put(io,node(ni,j),5);
      put(io,"     ");
      put(io,node(ni,j + 1),5);
      put(io,"     ");
      put(io,area,2,6,3);
      new_line(io);
    end loop;

    -- element 12
    new_line(io);
    new_line(io);
    put(io,"elem2      node      node");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- element 21
    new_line(io);
    new_line(io);
    put(io,"elem1      node      node      node         thickness");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,ns * ni * ni,5);
    new_line(io);

    new_line(io);
    e := 0;

    for j in 0 .. ns - 1 loop
      e := e + 1;

      put(io,e,5);
      put(io,"     ");
      put(io,node(0,0),5);
      put(io,"     ");
      put(io,node(1,j),5);
      put(io,"     ");
      put(io,node(1,j + 1),5);
      put(io,"     ");
      put(io,thickness,2,6,3);
      new_line(io);
    end loop;

    for i in 2 .. ni loop
      for j in 0 .. i * ns - 1 loop
        k := j / i;
        e := e + 1;

        put(io,e,5);
        put(io,"     ");
        put(io,node(i - 1,j - k),5);
        put(io,"     ");
        put(io,node(i,j),5);
        put(io,"     ");
        put(io,node(i,j + 1),5);
        put(io,"     ");
        put(io,thickness,2,6,3);
        new_line(io);
      end loop;
    end loop;

    for i in 1 .. ni - 1 loop
      for j in 0 .. i * ns - 1 loop
        k := j / i;
        e := e + 1;

        put(io,e,5);
        put(io,"     ");
        put(io,node(i + 1,j + 1 + k),5);
        put(io,"     ");
        put(io,node(i,j + 1),5);
        put(io,"     ");
        put(io,node(i,j),5);
        put(io,"     ");
        put(io,thickness,2,6,3);
        new_line(io);
      end loop;
    end loop;

    -- element 22
    new_line(io);
    new_line(io);
    put(io,"elem2      node      node      node");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- element 31
    new_line(io);
    new_line(io);
    put(io,"elem1      node      node      node      node");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- element 32
    new_line(io);
    new_line(io);
    put(io,"elem2      node      node      node      node");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- known displacements
    new_line(io);
    new_line(io);
    put(io," node      axis             displ");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,3 * ns,5);
    new_line(io);

    new_line(io);
    for k in 0 .. ns - 1 loop
      glob := node(ni,k * ni);

      if k rem 2 = 0 then
        put(io,glob,5);
        put(io,"         1     ");
        put(io,dr1 * cost(k),2,6,3);
        new_line(io);

        put(io,glob,5);
        put(io,"         2     ");
        put(io,dr1 * sint(k),2,6,3);
        new_line(io);

        put(io,glob,5);
        put(io,"         3     ");
        put(io,dz1,2,6,3);
        new_line(io);
      else
        put(io,glob,5);
        put(io,"         1     ");
        put(io,dr2 * cost(k),2,6,3);
        new_line(io);

        put(io,glob,5);
        put(io,"         2     ");
        put(io,dr2 * sint(k),2,6,3);
        new_line(io);

        put(io,glob,5);
        put(io,"         3     ");
        put(io,dz2,2,6,3);
        new_line(io);
      end if;
    end loop;

    close(io);
  end input;

  procedure autocad is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io;

    k  : integer;
    x  : vec3;
    io : file_type;

    procedure putelem21 (i1,j1,i2,j2,i3,j3 : in integer) is

    begin
      put(io,"3dface ");

      vertex(i1,j1,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      vertex(i2,j2,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      vertex(i3,j3,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      put(io," ");
      new_line(io);
    end putelem21;

  begin
    create(io, out_file, "polygon.scr");

    -- set running object snap mode to NONE
    put(io,"osmode 0");
    new_line(io);

    -- element 21
    put(io,"-color");
    new_line(io);
    put(io,"green");
    new_line(io);

    put(io,"layer make Initial_21 ");
    new_line(io);

    for j in 0 .. ns - 1 loop
      putelem21(0,0,1,j,1,j + 1);
    end loop;

    for i in 2 .. ni loop
      for j in 0 .. i * ns - 1 loop
        k := j / i;
        putelem21(i - 1,j - k,i,j,i,j + 1);
      end loop;
    end loop;

    for i in 1 .. ni - 1 loop
      for j in 0 .. i * ns - 1 loop
        k := j / i;
        putelem21(i + 1,j + 1 + k,i,j + 1,i,j);
      end loop;
    end loop;

    -- element 11
    put(io,"-color");
    new_line(io);
    put(io,"red");
    new_line(io);

    put(io,"layer make Initial_11 ");
    new_line(io);

    for j in 0 .. ni * ns - 1 loop
      put(io,"3dpoly ");

      vertex(ni,j,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      vertex(ni,j + 1,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      new_line(io);
    end loop;

    put(io,"-color");
    new_line(io);
    put(io,"white");
    new_line(io);

    put(io,"layer set 0 ");
    new_line(io);

    put(io,"zoom ext ext");
    new_line(io);

    close(io);
  end autocad;

begin
  initia;
  input;
  autocad;
end polygon;
